@@ -16,7 +16,7 @@
  */
 package org.apache.catalina;
 
-import java.util.Locale;
+import org.apache.tomcat.util.compat.JreVendor;
 
 /**
  * Global constants that are applicable to multiple packages within Catalina.
@@ -232,9 +232,6 @@ public final class Globals {
             org.apache.coyote.Constants.REMOTE_ADDR_ATTRIBUTE;
 
 
-    /**
-     *
-     */
     public static final String ASYNC_SUPPORTED_ATTR =
         "org.apache.catalina.ASYNC_SUPPORTED";
 
@@ -250,6 +247,13 @@ public final class Globals {
         "org.apache.catalina.parameter_parse_failed";
 
 
+    /**
+     * The reason that the parameter parsing failed.
+     */
+    public static final String PARAMETER_PARSE_FAILED_REASON_ATTR =
+            "org.apache.catalina.parameter_parse_failed_reason";
+
+
     /**
      * The master flag which controls strict servlet specification
      * compliance.
@@ -305,30 +309,9 @@ public final class Globals {
     public static final String JASPER_XML_BLOCK_EXTERNAL_INIT_PARAM =
             "org.apache.jasper.XML_BLOCK_EXTERNAL";
 
-    static {
-        /**
-         * There are a few places where Tomcat either accesses JVM internals
-         * (e.g. the memory leak protection) or where feature support varies
-         * between JVMs (e.g. SPNEGO). These flags exist to enable Tomcat to
-         * adjust its behaviour based on the vendor of the JVM. In an ideal
-         * world this code would not exist.
-         */
-        String vendor = System.getProperty("java.vendor", "");
-        vendor = vendor.toLowerCase(Locale.ENGLISH);
+    @Deprecated // Will be removed in Tomcat 9.0.x
+    public static final boolean IS_ORACLE_JVM = JreVendor.IS_ORACLE_JVM;
 
-        if (vendor.startsWith("oracle") || vendor.startsWith("sun")) {
-            IS_ORACLE_JVM = true;
-            IS_IBM_JVM = false;
-        } else if (vendor.contains("ibm")) {
-            IS_ORACLE_JVM = false;
-            IS_IBM_JVM = true;
-        } else {
-            IS_ORACLE_JVM = false;
-            IS_IBM_JVM = false;
-        }
-    }
-
-    public static final boolean IS_ORACLE_JVM;
-
-    public static final boolean IS_IBM_JVM;
+    @Deprecated // Will be removed in Tomcat 9.0.x
+    public static final boolean IS_IBM_JVM = JreVendor.IS_IBM_JVM;
 }
