@@ -21,7 +21,7 @@ import java.io.Reader;
 import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
-import java.util.HashMap;
+import java.util.concurrent.ConcurrentHashMap;
 
 import javax.servlet.ReadListener;
 
@@ -108,7 +108,7 @@ public class InputBuffer extends Reader
     /**
      * List of encoders.
      */
-    protected final HashMap<String,B2CConverter> encoders = new HashMap<>();
+    protected final ConcurrentHashMap<String,B2CConverter> encoders = new ConcurrentHashMap<>();
 
 
     /**
@@ -261,6 +261,10 @@ public class InputBuffer extends Reader
         // is not required.
         if (!coyoteRequest.isFinished() && isReady()) {
             coyoteRequest.action(ActionCode.DISPATCH_READ, null);
+            if (!ContainerThreadMarker.isContainerThread()) {
+                // Not on a container thread so need to execute the dispatch
+                coyoteRequest.action(ActionCode.DISPATCH_EXECUTE, null);
+            }
         }
     }
 
