@@ -540,17 +540,18 @@ public class ApplicationContext
      *  in the correct form
      */
     @Override
-    public URL getResource(String path)
-        throws MalformedURLException {
+    public URL getResource(String path) throws MalformedURLException {
 
-        if (path == null ||
-                !path.startsWith("/") && GET_RESOURCE_REQUIRE_SLASH)
-            throw new MalformedURLException(sm.getString(
-                    "applicationContext.requestDispatcher.iae", path));
+        String validatedPath = validateResourcePath(path);
+
+        if (validatedPath == null) {
+            throw new MalformedURLException(
+                    sm.getString("applicationContext.requestDispatcher.iae", path));
+        }
 
         WebResourceRoot resources = context.getResources();
         if (resources != null) {
-            return resources.getResource(path).getURL();
+            return resources.getResource(validatedPath).getURL();
         }
 
         return null;
@@ -568,21 +569,42 @@ public class ApplicationContext
     @Override
     public InputStream getResourceAsStream(String path) {
 
-        if (path == null)
-            return (null);
+        String validatedPath = validateResourcePath(path);
 
-        if (!path.startsWith("/") && GET_RESOURCE_REQUIRE_SLASH)
+        if (validatedPath == null) {
             return null;
+        }
 
         WebResourceRoot resources = context.getResources();
         if (resources != null) {
-            return resources.getResource(path).getInputStream();
+            return resources.getResource(validatedPath).getInputStream();
         }
 
         return null;
     }
 
 
+    /*
+     * Returns null if the input path is not valid or a path that will be
+     * acceptable to resoucres.getResource().
+     */
+    private String validateResourcePath(String path) {
+        if (path == null) {
+            return null;
+        }
+
+        if (!path.startsWith("/")) {
+            if (GET_RESOURCE_REQUIRE_SLASH) {
+                return null;
+            } else {
+                return "/" + path;
+            }
+        }
+
+        return path;
+    }
+
+
     /**
      * Return a Set containing the resource paths of resources member of the
      * specified collection. Each path will be a String starting with
@@ -782,17 +804,13 @@ public class ApplicationContext
             return;
         }
 
-        Object oldValue = null;
-        boolean replaced = false;
-
         // Add or replace the specified attribute
         // Check for read only attribute
         if (readOnlyAttributes.containsKey(name))
             return;
-        oldValue = attributes.get(name);
-        if (oldValue != null)
-            replaced = true;
-        attributes.put(name, value);
+
+        Object oldValue = attributes.put(name, value);
+        boolean replaced = oldValue != null;
 
         // Notify interested application event listeners
         Object listeners[] = context.getApplicationEventListeners();
