@@ -14,31 +14,32 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
+
 package org.apache.catalina.core;
 
 
 import java.io.IOException;
-import java.io.UnsupportedEncodingException;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Iterator;
 import java.util.Map;
 import java.util.NoSuchElementException;
 
+import javax.servlet.DispatcherType;
 import javax.servlet.RequestDispatcher;
+import javax.servlet.ServletContext;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletRequestWrapper;
 import javax.servlet.http.HttpSession;
 
 import org.apache.catalina.Context;
 import org.apache.catalina.Globals;
-import org.apache.catalina.Session;
 import org.apache.catalina.Manager;
-import org.apache.catalina.util.Enumerator;
-import org.apache.catalina.util.StringManager;
-import org.apache.catalina.util.ParameterMap;
-import org.apache.tomcat.util.buf.B2CConverter;
-import org.apache.tomcat.util.buf.MessageBytes;
-import org.apache.tomcat.util.http.Parameters;
+import org.apache.catalina.Session;
+import org.apache.catalina.util.RequestUtil;
 
 
 /**
@@ -55,9 +56,7 @@ import org.apache.tomcat.util.http.Parameters;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- *
  */
-
 class ApplicationHttpRequest extends HttpServletRequestWrapper {
 
 
@@ -68,18 +67,16 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * The set of attribute names that are special for request dispatchers.
      */
     protected static final String specials[] =
-    { Globals.INCLUDE_REQUEST_URI_ATTR, Globals.INCLUDE_CONTEXT_PATH_ATTR,
-      Globals.INCLUDE_SERVLET_PATH_ATTR, Globals.INCLUDE_PATH_INFO_ATTR,
-      Globals.INCLUDE_QUERY_STRING_ATTR, Globals.FORWARD_REQUEST_URI_ATTR,
-      Globals.FORWARD_CONTEXT_PATH_ATTR, Globals.FORWARD_SERVLET_PATH_ATTR,
-      Globals.FORWARD_PATH_INFO_ATTR, Globals.FORWARD_QUERY_STRING_ATTR };
-
-
-    /**
-     * The string manager for this package.
-     */
-    protected static StringManager sm =
-        StringManager.getManager(Constants.Package);
+    { RequestDispatcher.INCLUDE_REQUEST_URI,
+      RequestDispatcher.INCLUDE_CONTEXT_PATH,
+      RequestDispatcher.INCLUDE_SERVLET_PATH,
+      RequestDispatcher.INCLUDE_PATH_INFO,
+      RequestDispatcher.INCLUDE_QUERY_STRING,
+      RequestDispatcher.FORWARD_REQUEST_URI,
+      RequestDispatcher.FORWARD_CONTEXT_PATH,
+      RequestDispatcher.FORWARD_SERVLET_PATH,
+      RequestDispatcher.FORWARD_PATH_INFO,
+      RequestDispatcher.FORWARD_QUERY_STRING };
 
 
     // ----------------------------------------------------------- Constructors
@@ -107,7 +104,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     /**
      * The context for this request.
      */
-    protected Context context = null;
+    protected final Context context;
 
 
     /**
@@ -117,32 +114,23 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
 
 
     /**
-     * If this request is cross context, since this changes session accesss
+     * If this request is cross context, since this changes session access
      * behavior.
      */
-    protected boolean crossContext = false;
+    protected final boolean crossContext;
 
 
     /**
      * The current dispatcher type.
      */
-    protected Object dispatcherType = null;
-
-
-    /**
-     * Descriptive information about this implementation.
-     */
-    protected static final String info =
-        "org.apache.catalina.core.ApplicationHttpRequest/1.0";
+    protected DispatcherType dispatcherType = null;
 
 
     /**
      * The request parameters for this request.  This is initialized from the
-     * wrapped request.
-     * It is a {@code Map<String,String[]>}, as documented in
-     * ServletRequest.getParameterMap().
+     * wrapped request, but updates are allowed.
      */
-    protected Map parameters = null;
+    protected Map<String, String[]> parameters = null;
 
 
     /**
@@ -196,17 +184,26 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     /**
      * Special attributes.
      */
-    protected Object[] specialAttributes = new Object[specials.length];
+    protected final Object[] specialAttributes = new Object[specials.length];
 
 
     // ------------------------------------------------- ServletRequest Methods
 
+    @Override
+    public ServletContext getServletContext() {
+        if (context == null) {
+            return null;
+        }
+        return context.getServletContext();
+    }
+
 
     /**
      * Override the <code>getAttribute()</code> method of the wrapped request.
      *
      * @param name Name of the attribute to retrieve
      */
+    @Override
     public Object getAttribute(String name) {
 
         if (name.equals(Globals.DISPATCHER_TYPE_ATTR)) {
@@ -241,7 +238,8 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getAttributeNames()</code> method of the wrapped
      * request.
      */
-    public Enumeration getAttributeNames() {
+    @Override
+    public Enumeration<String> getAttributeNames() {
         return (new AttributeNamesEnumerator());
     }
 
@@ -252,6 +250,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param name Name of the attribute to remove
      */
+    @Override
     public void removeAttribute(String name) {
 
         if (!removeSpecial(name))
@@ -267,10 +266,11 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * @param name Name of the attribute to set
      * @param value Value of the attribute to set
      */
+    @Override
     public void setAttribute(String name, Object value) {
 
         if (name.equals(Globals.DISPATCHER_TYPE_ATTR)) {
-            dispatcherType = value;
+            dispatcherType = (DispatcherType)value;
             return;
         } else if (name.equals(Globals.DISPATCHER_REQUEST_PATH_ATTR)) {
             requestDispatcherPath = value;
@@ -290,6 +290,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param path Path of the resource to be wrapped
      */
+    @Override
     public RequestDispatcher getRequestDispatcher(String path) {
 
         if (context == null)
@@ -303,7 +304,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
 
         // Convert a request-relative path to a context-relative one
         String servletPath =
-            (String) getAttribute(Globals.INCLUDE_SERVLET_PATH_ATTR);
+            (String) getAttribute(RequestDispatcher.INCLUDE_SERVLET_PATH);
         if (servletPath == null)
             servletPath = getServletPath();
 
@@ -330,6 +331,16 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     }
 
 
+    /**
+     * Override the getDispatcherType() method of the wrapped request.
+     *
+     */
+    @Override
+    public DispatcherType getDispatcherType() {
+        return dispatcherType;
+    }
+
+
     // --------------------------------------------- HttpServletRequest Methods
 
 
@@ -337,6 +348,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getContextPath()</code> method of the wrapped
      * request.
      */
+    @Override
     public String getContextPath() {
 
         return (this.contextPath);
@@ -349,19 +361,16 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param name Name of the requested parameter
      */
+    @Override
     public String getParameter(String name) {
 
-	parseParameters();
+        parseParameters();
 
-        Object value = parameters.get(name);
-        if (value == null)
-            return (null);
-        else if (value instanceof String[])
-            return (((String[]) value)[0]);
-        else if (value instanceof String)
-            return ((String) value);
-        else
-            return (value.toString());
+        String[] value = parameters.get(name);
+        if (value == null) {
+            return null;
+        }
+        return value[0];
 
     }
 
@@ -370,9 +379,10 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getParameterMap()</code> method of the
      * wrapped request.
      */
-    public Map getParameterMap() {
+    @Override
+    public Map<String, String[]> getParameterMap() {
 
-	parseParameters();
+        parseParameters();
         return (parameters);
 
     }
@@ -382,11 +392,11 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getParameterNames()</code> method of the
      * wrapped request.
      */
-    public Enumeration getParameterNames() {
-
-	parseParameters();
-        return (new Enumerator(parameters.keySet()));
+    @Override
+    public Enumeration<String> getParameterNames() {
 
+        parseParameters();
+        return Collections.enumeration(parameters.keySet());
     }
 
 
@@ -396,23 +406,11 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param name Name of the requested parameter
      */
+    @Override
     public String[] getParameterValues(String name) {
 
-	parseParameters();
-        Object value = parameters.get(name);
-        if (value == null)
-            return ((String[]) null);
-        else if (value instanceof String[])
-            return ((String[]) value);
-        else if (value instanceof String) {
-            String values[] = new String[1];
-            values[0] = (String) value;
-            return (values);
-        } else {
-            String values[] = new String[1];
-            values[0] = value.toString();
-            return (values);
-        }
+        parseParameters();
+        return parameters.get(name);
 
     }
 
@@ -420,6 +418,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     /**
      * Override the <code>getPathInfo()</code> method of the wrapped request.
      */
+    @Override
     public String getPathInfo() {
 
         return (this.pathInfo);
@@ -427,10 +426,25 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     }
 
 
+    /**
+     * Override the <code>getPathTranslated()</code> method of the wrapped
+     * request.
+     */
+    @Override
+    public String getPathTranslated() {
+        if (getPathInfo() == null || getServletContext() == null) {
+            return null;
+        }
+
+        return getServletContext().getRealPath(getPathInfo());
+    }
+
+
     /**
      * Override the <code>getQueryString()</code> method of the wrapped
      * request.
      */
+    @Override
     public String getQueryString() {
 
         return (this.queryString);
@@ -442,6 +456,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getRequestURI()</code> method of the wrapped
      * request.
      */
+    @Override
     public String getRequestURI() {
 
         return (this.requestURI);
@@ -453,6 +468,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getRequestURL()</code> method of the wrapped
      * request.
      */
+    @Override
     public StringBuffer getRequestURL() {
 
         StringBuffer url = new StringBuffer();
@@ -480,6 +496,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getServletPath()</code> method of the wrapped
      * request.
      */
+    @Override
     public String getServletPath() {
 
         return (this.servletPath);
@@ -491,6 +508,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Return the session associated with this Request, creating one
      * if necessary.
      */
+    @Override
     public HttpSession getSession() {
         return (getSession(true));
     }
@@ -502,6 +520,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param create Create a new session if one does not exist
      */
+    @Override
     public HttpSession getSession(boolean create) {
 
         if (crossContext) {
@@ -559,6 +578,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * @return true if the request specifies a JSESSIONID that is valid within
      * the context of this ApplicationHttpRequest, false otherwise.
      */
+    @Override
     public boolean isRequestedSessionIdValid() {
 
         if (crossContext) {
@@ -575,7 +595,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
             try {
                 session = manager.findSession(requestedSessionId);
             } catch (IOException e) {
-                session = null;
+                // Ignore
             }
             if ((session != null) && session.isValid()) {
                 return (true);
@@ -603,10 +623,23 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
 
 
     /**
-     * Return descriptive information about this implementation.
+     * Perform a shallow copy of the specified Map, and return the result.
+     *
+     * @param orig Origin Map to be copied
      */
-    public String getInfo() {
-        return info;
+    Map<String, String[]> copyMap(Map<String, String[]> orig) {
+
+        if (orig == null) {
+            return (new HashMap<>());
+        }
+        HashMap<String, String[]> dest = new HashMap<>();
+
+        for (Map.Entry<String, String[]> entry : orig.entrySet()) {
+            dest.put(entry.getKey(), entry.getValue());
+        }
+
+        return (dest);
+
     }
 
 
@@ -656,7 +689,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         super.setRequest(request);
 
         // Initialize the attributes for this request
-        dispatcherType = request.getAttribute(Globals.DISPATCHER_TYPE_ATTR);
+        dispatcherType = (DispatcherType)request.getAttribute(Globals.DISPATCHER_TYPE_ATTR);
         requestDispatcherPath =
             request.getAttribute(Globals.DISPATCHER_REQUEST_PATH_ATTR);
 
@@ -702,14 +735,13 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      */
     void parseParameters() {
 
-	if (parsedParams) {
-	    return;
-	}
+        if (parsedParams) {
+            return;
+        }
 
-        parameters = new ParameterMap<String,String[]>();
-        parameters.putAll(getRequest().getParameterMap());
+        parameters = new HashMap<>();
+        parameters = copyMap(getRequest().getParameterMap());
         mergeParameters();
-        ((ParameterMap<String,String[]>) parameters).setLocked(true);
         parsedParams = true;
     }
 
@@ -724,10 +756,8 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         this.queryParamString = queryString;
     }
 
-
     // ------------------------------------------------------ Protected Methods
 
-
     /**
      * Is this attribute name one of the special ones that is added only for
      * included servlets?
@@ -801,29 +831,29 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      */
     protected String[] mergeValues(Object values1, Object values2) {
 
-        ArrayList<String> results = new ArrayList<String>();
+        ArrayList<Object> results = new ArrayList<>();
 
         if (values1 == null) {
             // Skip - nothing to merge
-        } else if (values1 instanceof String[]) {
+        } else if (values1 instanceof String)
+            results.add(values1);
+        else if (values1 instanceof String[]) {
             String values[] = (String[]) values1;
-            for (int i = 0; i < values.length; i++) {
+            for (int i = 0; i < values.length; i++)
                 results.add(values[i]);
-            }
-        } else { // String
+        } else
             results.add(values1.toString());
-        }
 
         if (values2 == null) {
             // Skip - nothing to merge
-        } else if (values2 instanceof String[]) {
+        } else if (values2 instanceof String)
+            results.add(values2);
+        else if (values2 instanceof String[]) {
             String values[] = (String[]) values2;
-            for (int i = 0; i < values.length; i++) {
+            for (int i = 0; i < values.length; i++)
                 results.add(values[i]);
-            }
-        } else { // String
+        } else
             results.add(values2.toString());
-        }
 
         String values[] = new String[results.size()];
         return results.toArray(values);
@@ -845,38 +875,25 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         if ((queryParamString == null) || (queryParamString.length() < 1))
             return;
 
-        // Parse the query string from the dispatch target
-        Parameters paramParser = new Parameters();
-        MessageBytes queryMB = MessageBytes.newInstance();
-        queryMB.setString(queryParamString);
-
+        HashMap<String, String[]> queryParameters = new HashMap<>();
         String encoding = getCharacterEncoding();
-        // No need to process null value, as ISO-8859-1 is the default encoding
-        // in MessageBytes.toBytes().
-        if (encoding != null) {
-            try {
-                queryMB.setCharset(B2CConverter.getCharset(encoding));
-            } catch (UnsupportedEncodingException ignored) {
-                // Fall-back to ISO-8859-1
-            }
-        }
-
-        paramParser.setQuery(queryMB);
-        paramParser.setQueryStringEncoding(encoding);
-        paramParser.handleQueryParameters();
-
-        // Insert the additional parameters from the dispatch target
-        Enumeration<String> dispParamNames = paramParser.getParameterNames();
-        while (dispParamNames.hasMoreElements()) {
-            String dispParamName = dispParamNames.nextElement();
-            String[] dispParamValues = paramParser.getParameterValues(dispParamName);
-            String[] originalValues = (String[]) parameters.get(dispParamName);
-            if (originalValues == null) {
-                parameters.put(dispParamName, dispParamValues);
+        if (encoding == null)
+            encoding = "ISO-8859-1";
+        RequestUtil.parseParameters(queryParameters, queryParamString,
+                encoding);
+        Iterator<String> keys = parameters.keySet().iterator();
+        while (keys.hasNext()) {
+            String key = keys.next();
+            Object value = queryParameters.get(key);
+            if (value == null) {
+                queryParameters.put(key, parameters.get(key));
                 continue;
             }
-            parameters.put(dispParamName, mergeValues(dispParamValues, originalValues));
+            queryParameters.put
+                (key, mergeValues(value, parameters.get(key)));
         }
+        parameters = queryParameters;
+
     }
 
 
@@ -887,28 +904,33 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Utility class used to expose the special attributes as being available
      * as request attributes.
      */
-    protected class AttributeNamesEnumerator implements Enumeration {
+    protected class AttributeNamesEnumerator implements Enumeration<String> {
 
         protected int pos = -1;
-        protected int last = -1;
-        protected Enumeration parentEnumeration = null;
+        protected final int last;
+        protected final Enumeration<String> parentEnumeration;
         protected String next = null;
 
         public AttributeNamesEnumerator() {
+            int last = -1;
             parentEnumeration = getRequest().getAttributeNames();
-            for (int i = 0; i < specialAttributes.length; i++) {
+            for (int i = specialAttributes.length - 1; i >= 0; i--) {
                 if (getAttribute(specials[i]) != null) {
                     last = i;
+                    break;
                 }
             }
+            this.last = last;
         }
 
+        @Override
         public boolean hasMoreElements() {
             return ((pos != last) || (next != null)
                     || ((next = findNext()) != null));
         }
 
-        public Object nextElement() {
+        @Override
+        public String nextElement() {
             if (pos != last) {
                 for (int i = pos + 1; i <= last; i++) {
                     if (getAttribute(specials[i]) != null) {
@@ -929,7 +951,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         protected String findNext() {
             String result = null;
             while ((result == null) && (parentEnumeration.hasMoreElements())) {
-                String current = (String) parentEnumeration.nextElement();
+                String current = parentEnumeration.nextElement();
                 if (!isSpecial(current)) {
                     result = current;
                 }
