@@ -22,6 +22,7 @@ import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletResponse;
 
+import org.apache.catalina.Container;
 import org.apache.catalina.Wrapper;
 import org.apache.catalina.comet.CometEvent;
 import org.apache.catalina.connector.Request;
@@ -46,18 +47,13 @@ final class StandardContextValve extends ValveBase {
 
 
     /**
-     * The descriptive information related to this implementation.
-     */
-    private static final String info =
-        "org.apache.catalina.core.StandardContextValve/1.0";
-
-
-    /**
-     * Return descriptive information about this Valve implementation.
+     * Cast to a StandardContext right away, as it will be needed later.
+     *
+     * @see org.apache.catalina.Contained#setContainer(org.apache.catalina.Container)
      */
     @Override
-    public String getInfo() {
-        return (info);
+    public void setContainer(Container container) {
+        super.setContainer(container);
     }
 
 
