@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -21,7 +21,6 @@ import java.io.IOException;
 import javax.servlet.ServletException;
 
 import org.apache.catalina.Container;
-import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
 import org.apache.catalina.Host;
 import org.apache.catalina.LifecycleException;
@@ -49,43 +48,34 @@ import org.apache.tomcat.util.res.StringManager;
  * nodes. After all that, the session stickiness will work directly to the
  * backup node and the traffic will not go back to the failed node after it is
  * restarted!
- * 
- * <p>
- * For this valve to function correctly, so that all nodes of the cluster
- * receive the sessionid change notifications that it generates, the following
- * ClusterListener MUST be configured at all nodes of the cluster:
- * {@link org.apache.catalina.ha.session.JvmRouteSessionIDBinderListener
- * JvmRouteSessionIDBinderListener} since Tomcat 5.5.10, and both
- * JvmRouteSessionIDBinderListener and JvmRouteSessionIDBinderLifecycleListener
- * for earlier versions of Tomcat.
- * 
+ *
  * <p>
  * Add this Valve to your host definition at conf/server.xml .
- * 
- * Since 5.5.10 as direct cluster valve:<br/>
- * 
+ *
+ * Since 5.5.10 as direct cluster valve:<br>
+ *
  * <pre>
  *  &lt;Cluster&gt;
- *  &lt;Valve className=&quot;org.apache.catalina.ha.session.JvmRouteBinderValve&quot; /&gt;  
+ *  &lt;Valve className=&quot;org.apache.catalina.ha.session.JvmRouteBinderValve&quot; /&gt;
  *  &lt;/Cluster&gt;
  * </pre>
- * 
- * <br />
- * Before 5.5.10 as Host element:<br/>
- * 
+ *
+ * <br>
+ * Before 5.5.10 as Host element:<br>
+ *
  * <pre>
  *  &lt;Host&gt;
- *  &lt;Valve className=&quot;org.apache.catalina.ha.session.JvmRouteBinderValve&quot; /&gt;  
+ *  &lt;Valve className=&quot;org.apache.catalina.ha.session.JvmRouteBinderValve&quot; /&gt;
  *  &lt;/Host&gt;
  * </pre>
- * 
- * <em>A Trick:</em><br/>
+ *
+ * <em>A Trick:</em><br>
  * You can enable this mod_jk turnover mode via JMX before you drop a node to
  * all backup nodes! Set enable true on all JvmRouteBinderValve backups, disable
  * worker at mod_jk and then drop node and restart it! Then enable mod_jk worker
  * and disable JvmRouteBinderValves again. This use case means that only
  * requested sessions are migrated.
- * 
+ *
  * @author Peter Rossbach
  */
 public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
@@ -94,11 +84,6 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
     public static final org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory
             .getLog(JvmRouteBinderValve.class);
 
-    /**
-     * The descriptive information about this implementation.
-     */
-    protected static final String info = "org.apache.catalina.ha.session.JvmRouteBinderValve/1.2";
-
     //------------------------------------------------------ Constructor
     public JvmRouteBinderValve() {
         super(true);
@@ -122,7 +107,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
     protected boolean enabled = true;
 
     /**
-     * number of session that no at this tomcat instance hosted
+     * number of session that no at this tomcat instanz hosted
      */
     protected long numberOfSessions = 0;
 
@@ -131,19 +116,9 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
 
     /*--Logic---------------------------------------------------*/
 
-    /**
-     * Return descriptive information about this implementation.
-     */
-    @Override
-    public String getInfo() {
-
-        return (info);
-
-    }
-
     /**
      * set session id attribute to failed node for request.
-     * 
+     *
      * @return Returns the sessionIdAttribute.
      */
     public String getSessionIdAttribute() {
@@ -152,7 +127,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
 
     /**
      * get name of failed request session attribute
-     * 
+     *
      * @param sessionIdAttribute
      *            The sessionIdAttribute to set.
      */
@@ -184,7 +159,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
 
     /**
      * Detect possible the JVMRoute change at cluster backup node..
-     * 
+     *
      * @param request
      *            tomcat request being processed
      * @param response
@@ -202,7 +177,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
                  request.getContext() != null &&
                  request.getContext().getDistributable() &&
                  !request.isAsyncDispatching()) {
-             // valve cluster can access manager - other cluster handle turnover 
+             // valve cluster can access manager - other cluster handle turnover
              // at host level - hopefully!
              Manager manager = request.getContext().getManager();
 
@@ -211,8 +186,9 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
                        && getCluster() != null
                        && getCluster().getManager(((ClusterManager)manager).getName()) != null)
                      ||
-                     (manager instanceof PersistentManager)))
-                 handlePossibleTurnover(request);
+                     (manager instanceof PersistentManager))) {
+                handlePossibleTurnover(request);
+            }
         }
         // Pass this request on to the next valve in our pipeline
         getNext().invoke(request, response);
@@ -220,7 +196,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
 
     /**
      * handle possible session turn over.
-     * 
+     *
      * @see JvmRouteBinderValve#handleJvmRoute(Request, String, String)
      * @param request current request
      */
@@ -230,8 +206,9 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
             long t1 = System.currentTimeMillis();
             String jvmRoute = getLocalJvmRoute(request);
             if (jvmRoute == null) {
-                if (log.isDebugEnabled())
+                if (log.isDebugEnabled()) {
                     log.debug(sm.getString("jvmRoute.missingJvmRouteAttribute"));
+                }
                 return;
             }
             handleJvmRoute( request, sessionID, jvmRoute);
@@ -245,30 +222,32 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
 
     /**
      * get jvmroute from engine
-     * 
+     *
      * @param request current request
      * @return return jvmRoute from ManagerBase or null
      */
     protected String getLocalJvmRoute(Request request) {
         Manager manager = getManager(request);
-        if(manager instanceof ManagerBase)
+        if(manager instanceof ManagerBase) {
             return ((ManagerBase) manager).getJvmRoute();
+        }
         return null ;
     }
 
     /**
      * get ClusterManager
-     * 
+     *
      * @param request current request
      * @return manager or null
      */
     protected Manager getManager(Request request) {
         Manager manager = request.getContext().getManager();
         if (log.isDebugEnabled()) {
-            if(manager != null)
+            if(manager != null) {
                 log.debug(sm.getString("jvmRoute.foundManager", manager,  request.getContext().getName()));
-            else 
+            } else {
                 log.debug(sm.getString("jvmRoute.notFoundManager", request.getContext().getName()));
+            }
         }
         return manager;
     }
@@ -280,7 +259,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
     public CatalinaCluster getCluster() {
         return cluster;
     }
-    
+
     /**
      * @param cluster The cluster to set.
      */
@@ -288,12 +267,12 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
     public void setCluster(CatalinaCluster cluster) {
         this.cluster = cluster;
     }
-    
+
     /**
      * Handle jvmRoute stickiness after tomcat instance failed. After this
      * correction a new Cookie send to client with new jvmRoute and the
      * SessionID change propagate to the other cluster nodes.
-     * 
+     *
      * @param request current request
      * @param sessionId
      *            request SessionID from Cookie
@@ -304,7 +283,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
             Request request, String sessionId, String localJvmRoute) {
         // get requested jvmRoute.
         String requestJvmRoute = null;
-        int index = sessionId.indexOf('.');
+        int index = sessionId.indexOf(".");
         if (index > 0) {
             requestJvmRoute = sessionId
                     .substring(index + 1, sessionId.length());
@@ -347,7 +326,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
 
     /**
      * change session id and send to all cluster nodes
-     * 
+     *
      * @param request current request
      * @param sessionId
      *            original session id
@@ -359,21 +338,13 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
     protected void changeSessionID(Request request, String sessionId,
             String newSessionID, Session catalinaSession) {
         fireLifecycleEvent("Before session migration", catalinaSession);
-        catalinaSession.setId(newSessionID, false);
-        // FIXME: Why we remove change data from other running request?
-        // setId also trigger resetDeltaRequest!!
-        if (catalinaSession instanceof DeltaSession)
-            ((DeltaSession) catalinaSession).resetDeltaRequest();
+        catalinaSession.getManager().changeSessionId(catalinaSession, newSessionID);
         changeRequestSessionID(request, sessionId, newSessionID);
-
-        // now sending the change to all other clusternodes!
-        sendSessionIDClusterBackup(request,sessionId, newSessionID);
-
         fireLifecycleEvent("After session migration", catalinaSession);
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("jvmRoute.changeSession", sessionId,
                     newSessionID));
-        }   
+        }
     }
 
     /**
@@ -396,30 +367,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
             request.setAttribute(sessionIdAttribute, sessionId);
         }
     }
-    
-    /**
-     * Send the changed Sessionid to all clusternodes.
-     * 
-     * @see JvmRouteSessionIDBinderListener#messageReceived(
-     *            org.apache.catalina.ha.ClusterMessage)
-     * @param sessionId
-     *            current failed sessionid
-     * @param newSessionID
-     *            new session id, bind to the new cluster node
-     */
-    protected void sendSessionIDClusterBackup(Request request, String sessionId,
-            String newSessionID) {
-        CatalinaCluster c = getCluster();
-        if (c != null && !(getManager(request) instanceof BackupManager)) {
-            SessionIDMessage msg = new SessionIDMessage();
-            msg.setOrignalSessionID(sessionId);
-            msg.setBackupSessionID(newSessionID);
-            Context context = request.getContext();
-            msg.setContextName(context.getName());
-            msg.setHost(context.getParent().getName());
-            c.send(msg);
-        }
-    }
+
 
     /**
      * Start this component and implement the requirements
@@ -430,14 +378,15 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
      */
     @Override
     protected synchronized void startInternal() throws LifecycleException {
-        
+
         if (cluster == null) {
             Container hostContainer = getContainer();
             // compatibility with JvmRouteBinderValve version 1.1
             // ( setup at context.xml or context.xml.default )
             if (!(hostContainer instanceof Host)) {
-                if (log.isWarnEnabled())
+                if (log.isWarnEnabled()) {
                     log.warn(sm.getString("jvmRoute.configure.warn"));
+                }
                 hostContainer = hostContainer.getParent();
             }
             if (hostContainer instanceof Host
@@ -451,17 +400,18 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
                 }
             }
         }
-        
+
         if (log.isInfoEnabled()) {
             log.info(sm.getString("jvmRoute.valve.started"));
-            if (cluster == null)
+            if (cluster == null) {
                 log.info(sm.getString("jvmRoute.noCluster"));
+            }
         }
 
         super.startInternal();
     }
 
-    
+
     /**
      * Stop this component and implement the requirements
      * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
@@ -476,8 +426,9 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
 
         cluster = null;
         numberOfSessions = 0;
-        if (log.isInfoEnabled())
+        if (log.isInfoEnabled()) {
             log.info(sm.getString("jvmRoute.valve.stopped"));
+        }
 
     }
 
