@@ -553,7 +553,7 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
     }
 
     protected void checkDefaults() {
-        if ( clusterListeners.size() == 0 ) {
+        if ( clusterListeners.size() == 0 && managerTemplate instanceof DeltaManager ) {
             addClusterListener(new ClusterSessionListener());
         }
         if ( valves.size() == 0 ) {
