@@ -293,8 +293,16 @@ public class DataSourceRealm extends RealmBase {
 
         String dbCredentials = getPassword(dbConnection, username);
 
+        if (credentials == null || dbCredentials == null) {
+            if (containerLog.isTraceEnabled())
+                containerLog.trace(
+                    sm.getString("dataSourceRealm.authenticateFailure",
+                                 username));
+            return null;
+        }
+
         // Validate the user's credentials
-        boolean validated = compareCredentials(credentials, dbCredentials);
+        boolean validated = getCredentialHandler().matches(credentials, dbCredentials);
 
         if (validated) {
             if (containerLog.isTraceEnabled())
