@@ -14,10 +14,8 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.catalina.tribes.membership;
 
-
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -43,24 +41,23 @@ public class Membership implements Cloneable {
     private final Object membersLock = new Object();
 
     /**
-     * The name of this membership, has to be the same as the name for the local
-     * member
+     * The local member.
      */
     protected final Member local;
 
     /**
      * A map of all the members in the cluster.
      */
-    protected HashMap<Member, MbrEntry> map = new HashMap<>();
+    protected HashMap<Member, MbrEntry> map = new HashMap<>(); // Guarded by membersLock
 
     /**
      * A list of all the members in the cluster.
      */
-    protected Member[] members = EMPTY_MEMBERS;
+    protected volatile Member[] members = EMPTY_MEMBERS; // Guarded by membersLock
 
     /**
-      * sort members by alive time
-      */
+     * Comparator for sorting members by alive time.
+     */
     protected final Comparator<Member> memberComparator;
 
     @Override
@@ -71,7 +68,7 @@ public class Membership implements Cloneable {
             final HashMap<Member, MbrEntry> tmpclone = (HashMap<Member, MbrEntry>) map.clone();
             clone.map = tmpclone;
             clone.members = new Member[members.length];
-            System.arraycopy(members,0,clone.members,0,members.length);
+            System.arraycopy(members, 0, clone.members, 0, members.length);
             return clone;
         }
     }
@@ -95,16 +92,21 @@ public class Membership implements Cloneable {
 
     public Membership(Member local, Comparator<Member> comp, boolean includeLocal) {
         this.local = local;
-        if ( includeLocal ) addMember(local);
         this.memberComparator = comp;
+        if (includeLocal) {
+            addMember(local);
+        }
     }
+
     /**
-     * Reset the membership and start over fresh.
-     * Ie, delete all the members and wait for them to ping again and join this membership
+     * Reset the membership and start over fresh. i.e., delete all the members
+     * and wait for them to ping again and join this membership.
      */
-    public synchronized void reset() {
-        map.clear();
-        members = EMPTY_MEMBERS ;
+    public void reset() {
+        synchronized (membersLock) {
+            map.clear();
+            members = EMPTY_MEMBERS ;
+        }
     }
 
     /**
@@ -114,48 +116,59 @@ public class Membership implements Cloneable {
      * @return - true if this member is new to the cluster, false otherwise.<br>
      * - false if this member is the local member or updated.
      */
-    public synchronized boolean memberAlive(Member member) {
+    public boolean memberAlive(Member member) {
+        // Ignore ourselves
+        if (member.equals(local)) {
+            return false;
+        }
+
         boolean result = false;
-        //ignore ourselves
-        if (  member.equals(local) ) return result;
-
-        //return true if the membership has changed
-        MbrEntry entry = map.get(member);
-        if ( entry == null ) {
-            entry = addMember(member);
-            result = true;
-       } else {
-            //update the member alive time
-            Member updateMember = entry.getMember() ;
-            if(updateMember.getMemberAliveTime() != member.getMemberAliveTime()) {
-                //update fields that can change
-                updateMember.setMemberAliveTime(member.getMemberAliveTime());
-                updateMember.setPayload(member.getPayload());
-                updateMember.setCommand(member.getCommand());
-                Arrays.sort(members, memberComparator);
+        synchronized (membersLock) {
+            MbrEntry entry = map.get(member);
+            if (entry == null) {
+                entry = addMember(member);
+                result = true;
+            } else {
+                // Update the member alive time
+                Member updateMember = entry.getMember();
+                if (updateMember.getMemberAliveTime() != member.getMemberAliveTime()) {
+                    // Update fields that can change
+                    updateMember.setMemberAliveTime(member.getMemberAliveTime());
+                    updateMember.setPayload(member.getPayload());
+                    updateMember.setCommand(member.getCommand());
+                    // Re-order. Can't sort in place since a call to
+                    // getMembers() may then receive an intermediate result.
+                    Member[] newMembers = new Member[members.length];
+                    System.arraycopy(members, 0, newMembers, 0, members.length);
+                    Arrays.sort(newMembers, memberComparator);
+                    members = newMembers;
+                }
             }
+            entry.accessed();
         }
-        entry.accessed();
         return result;
     }
 
     /**
      * Add a member to this component and sort array with memberComparator
+     *
      * @param member The member to add
+     *
+     * @return The member entry created for this new member.
      */
-    public synchronized MbrEntry addMember(Member member) {
-      synchronized (membersLock) {
-          MbrEntry entry = new MbrEntry(member);
-          if (!map.containsKey(member) ) {
-              map.put(member, entry);
-              Member results[] = new Member[members.length + 1];
-              for (int i = 0; i < members.length; i++) results[i] = members[i];
-              results[members.length] = member;
-              members = results;
-              Arrays.sort(members, memberComparator);
-          }
-          return entry;
-      }
+    public MbrEntry addMember(Member member) {
+        MbrEntry entry = new MbrEntry(member);
+        synchronized (membersLock) {
+            if (!map.containsKey(member) ) {
+                map.put(member, entry);
+                Member results[] = new Member[members.length + 1];
+                System.arraycopy(members, 0, results, 0, members.length);
+                results[members.length] = member;
+                Arrays.sort(results, memberComparator);
+                members = results;
+            }
+        }
+        return entry;
     }
 
     /**
@@ -164,8 +177,8 @@ public class Membership implements Cloneable {
      * @param member The member to remove
      */
     public void removeMember(Member member) {
-        map.remove(member);
         synchronized (membersLock) {
+            map.remove(member);
             int n = -1;
             for (int i = 0; i < members.length; i++) {
                 if (members[i] == member || members[i].equals(member)) {
@@ -177,8 +190,9 @@ public class Membership implements Cloneable {
             Member results[] = new Member[members.length - 1];
             int j = 0;
             for (int i = 0; i < members.length; i++) {
-                if (i != n)
+                if (i != n) {
                     results[j++] = members[i];
+                }
             }
             members = results;
         }
@@ -191,74 +205,82 @@ public class Membership implements Cloneable {
      * @param maxtime - the max time a member can remain unannounced before it is considered dead.
      * @return the list of expired members
      */
-    public synchronized Member[] expire(long maxtime) {
-        if(!hasMembers() )
-           return EMPTY_MEMBERS;
-
-        ArrayList<Member> list = null;
-        Iterator<MbrEntry> i = map.values().iterator();
-        while(i.hasNext()) {
-            MbrEntry entry = i.next();
-            if( entry.hasExpired(maxtime) ) {
-                if(list == null) // only need a list when members are expired (smaller gc)
-                    list = new java.util.ArrayList<>();
-                list.add(entry.getMember());
+    public Member[] expire(long maxtime) {
+        synchronized (membersLock) {
+            if (!hasMembers()) {
+               return EMPTY_MEMBERS;
             }
-        }
 
-        if(list != null) {
-            Member[] result = new Member[list.size()];
-            list.toArray(result);
-            for( int j=0; j<result.length; j++) {
-                removeMember(result[j]);
+            ArrayList<Member> list = null;
+            Iterator<MbrEntry> i = map.values().iterator();
+            while (i.hasNext()) {
+                MbrEntry entry = i.next();
+                if (entry.hasExpired(maxtime)) {
+                    if (list == null) {
+                        // Only need a list when members are expired (smaller gc)
+                        list = new java.util.ArrayList<>();
+                    }
+                    list.add(entry.getMember());
+                }
+            }
+
+            if (list != null) {
+                Member[] result = new Member[list.size()];
+                list.toArray(result);
+                for (int j=0; j<result.length; j++) {
+                    removeMember(result[j]);
+                }
+                return result;
+            } else {
+                return EMPTY_MEMBERS ;
             }
-            return result;
-        } else {
-            return EMPTY_MEMBERS ;
         }
     }
 
     /**
-     * Returning that service has members or not
+     * Returning that service has members or not.
+     *
+     * @return <code>true</code> if there are one or more members, otherwise
+     *         <code>false</code>
      */
     public boolean hasMembers() {
-        return members.length > 0 ;
+        return members.length > 0;
     }
 
 
     public Member getMember(Member mbr) {
-        if(hasMembers()) {
-            Member result = null;
-            for ( int i=0; i<this.members.length && result==null; i++ ) {
-                if ( members[i].equals(mbr) ) result = members[i];
-            }//for
-            return result;
-        } else {
-            return null;
+        Member[] members = this.members;
+        if (members.length > 0) {
+            for (int i = 0; i < members.length; i++) {
+                if (members[i].equals(mbr)) {
+                    return members[i];
+                }
+            }
         }
+        return null;
     }
 
     public boolean contains(Member mbr) {
-        return getMember(mbr)!=null;
+        return getMember(mbr) != null;
     }
 
     /**
-     * Returning a list of all the members in the membership
+     * Returning a list of all the members in the membership.
      * We not need a copy: add and remove generate new arrays.
+     *
+     * @return An array of the current members
      */
     public Member[] getMembers() {
-        if(hasMembers()) {
-            return members;
-        } else {
-            return EMPTY_MEMBERS;
-        }
+        return members;
     }
 
     /**
-     * get a copy from all member entries
+     * Get a copy from all member entries.
+     *
+     * @deprecated Unused. Will be removed in Tomcat 9.0.x.
      */
-    protected synchronized MbrEntry[] getMemberEntries()
-    {
+    @Deprecated
+    protected synchronized MbrEntry[] getMemberEntries() {
         MbrEntry[] result = new MbrEntry[map.size()];
         Iterator<Map.Entry<Member,MbrEntry>> i = map.entrySet().iterator();
         int pos = 0;
@@ -269,21 +291,21 @@ public class Membership implements Cloneable {
 
     // --------------------------------------------- Inner Class
 
-    private static class MemberComparator implements Comparator<Member>,
-            Serializable {
+    private static class MemberComparator implements Comparator<Member>, Serializable {
 
         private static final long serialVersionUID = 1L;
 
         @Override
         public int compare(Member m1, Member m2) {
-            //longer alive time, means sort first
+            // Longer alive time, means sort first
             long result = m2.getMemberAliveTime() - m1.getMemberAliveTime();
-            if (result < 0)
+            if (result < 0) {
                 return -1;
-            else if (result == 0)
+            } else if (result == 0) {
                 return 0;
-            else
+            } else {
                 return 1;
+            }
         }
     }
 
@@ -307,15 +329,21 @@ public class Membership implements Cloneable {
         }
 
         /**
-         * Return the actual Member object
+         * Obtain the member associated with this entry.
+         *
+         * @return The member for this entry.
          */
         public Member getMember() {
             return mbr;
         }
 
         /**
-         * Check if this dude has expired
+         * Check if this member has expired.
+         *
          * @param maxtime The time threshold
+         *
+         * @return <code>true</code> if the member has expired, otherwise
+         *         <code>false</false>
          */
         public boolean hasExpired(long maxtime) {
             long delta = System.currentTimeMillis() - lastHeardFrom;
