@@ -18,6 +18,7 @@ package org.apache.coyote;
 
 import java.io.IOException;
 import java.net.InetAddress;
+import java.nio.ByteBuffer;
 import java.util.Iterator;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.Executor;
@@ -644,9 +645,23 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
                         // Do nothing here, just wait for it to get recycled
                         // Don't do this for Comet we need to generate an end
                         // event (see BZ 54022)
-                    } else if (processor.isAsync() ||
-                            state == SocketState.ASYNC_END) {
+                    } else if (processor.isAsync()) {
                         state = processor.asyncDispatch(status);
+                    } else if (state == SocketState.ASYNC_END) {
+                        state = processor.asyncDispatch(status);
+                        // release() won't get called so in case this request
+                        // takes a long time to process remove the socket from
+                        // the waiting requests now else the async timeout will
+                        // fire
+                        getProtocol().endpoint.removeWaitingRequest(wrapper);
+                        if (state == SocketState.OPEN) {
+                            // There may be pipe-lined data to read. If the data
+                            // isn't processed now, execution will exit this
+                            // loop and call release() which will recycle the
+                            // processor (and input buffer) deleting any
+                            // pipe-lined data. To avoid this, process it now.
+                            state = processor.process(wrapper);
+                        }
                     } else if (processor.isComet()) {
                         state = processor.event(status);
                     } else if (processor.isUpgrade()) {
@@ -666,11 +681,13 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
                         // Get the HTTP upgrade handler
                         HttpUpgradeHandler httpUpgradeHandler =
                                 processor.getHttpUpgradeHandler();
+                        // Retrieve leftover input
+                        ByteBuffer leftoverInput = processor.getLeftoverInput();
                         // Release the Http11 processor to be re-used
                         release(wrapper, processor, false, false);
                         // Create the upgrade processor
                         processor = createUpgradeProcessor(
-                                wrapper, httpUpgradeHandler);
+                                wrapper, leftoverInput, httpUpgradeHandler);
                         // Mark the connection as upgraded
                         wrapper.setUpgraded(true);
                         // Associate with the processor with the connection
@@ -776,7 +793,7 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
                 Processor<S> processor, boolean socketClosing,
                 boolean addToPoller);
         protected abstract Processor<S> createUpgradeProcessor(
-                SocketWrapper<S> socket,
+                SocketWrapper<S> socket, ByteBuffer leftoverInput,
                 HttpUpgradeHandler httpUpgradeProcessor) throws IOException;
 
         protected void register(AbstractProcessor<S> processor) {
