@@ -436,15 +436,17 @@ class JspReader {
 
     /**
      * Skip until the given string is matched in the stream, but ignoring
-     * chars initially escaped by a '\'.
+     * chars initially escaped by a '\' and any EL expressions.
      * When returned, the context is positioned past the end of the match.
      *
      * @param s The String to match.
+     * @param ignoreEL <code>true</code> if something that looks like EL should
+     *                 not be treated as EL.
      * @return A non-null <code>Mark</code> instance (positioned immediately
      *         before the search string) if found, <strong>null</strong>
      *         otherwise.
      */
-    Mark skipUntilIgnoreEsc(String limit) {
+    Mark skipUntilIgnoreEsc(String limit, boolean ignoreEL) {
         Mark ret = mark();
         int limlen = limit.length();
         int ch;
@@ -454,6 +456,12 @@ class JspReader {
         for (ch = nextChar(ret) ; ch != -1 ; prev = ch, ch = nextChar(ret)) {
             if (ch == '\\' && prev == '\\') {
                 ch = 0;                // Double \ is not an escape char anymore
+            } else if (prev == '\\') {
+                continue;
+            } else if (!ignoreEL && (ch == '$' || ch == '#') && peekChar() == '{' ) {
+                // Move beyond the '{'
+                nextChar();
+                skipELExpression();
             } else if (ch == firstChar && prev != '\\') {
                 for (int i = 1 ; i < limlen ; i++) {
                     if (peekChar() == limit.charAt(i))
