@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -18,8 +18,8 @@ package org.apache.jasper.compiler;
 
 import java.io.CharArrayWriter;
 import java.io.FileNotFoundException;
+import java.util.Collection;
 import java.util.Iterator;
-import java.util.List;
 
 import javax.servlet.jsp.tagext.TagAttributeInfo;
 import javax.servlet.jsp.tagext.TagFileInfo;
@@ -29,6 +29,8 @@ import javax.servlet.jsp.tagext.TagLibraryInfo;
 import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
 import org.apache.jasper.util.UniqueAttributesImpl;
+import org.apache.tomcat.util.descriptor.tld.TldResourcePath;
+import org.apache.tomcat.util.scan.Jar;
 import org.xml.sax.Attributes;
 import org.xml.sax.helpers.AttributesImpl;
 
@@ -36,7 +38,7 @@ import org.xml.sax.helpers.AttributesImpl;
  * This class implements a parser for a JSP page (non-xml view). JSP page
  * grammar is included here for reference. The token '#' that appears in the
  * production indicates the current input token location in the production.
- * 
+ *
  * @author Kin-man Chung
  * @author Shawn Bayern
  * @author Mark Roth
@@ -44,25 +46,25 @@ import org.xml.sax.helpers.AttributesImpl;
 
 class Parser implements TagConstants {
 
-    private ParserController parserController;
+    private final ParserController parserController;
 
-    private JspCompilationContext ctxt;
+    private final JspCompilationContext ctxt;
 
-    private JspReader reader;
+    private final JspReader reader;
 
     private Mark start;
 
-    private ErrorDispatcher err;
+    private final ErrorDispatcher err;
 
     private int scriptlessCount;
 
-    private boolean isTagFile;
+    private final boolean isTagFile;
 
-    private boolean directivesOnly;
+    private final boolean directivesOnly;
 
-    private JarResource jarResource;
+    private final Jar jar;
 
-    private PageInfo pageInfo;
+    private final PageInfo pageInfo;
 
     // Virtual body content types, to make parsing a little easier.
     // These are not accessible from outside the parser.
@@ -77,16 +79,16 @@ class Parser implements TagConstants {
 
     /* System property that controls if the strict white space rules are
      * applied.
-     */ 
-    private static final boolean STRICT_WHITESPACE = Boolean.parseBoolean(
+     */
+    private static final boolean STRICT_WHITESPACE = Boolean.valueOf(
             System.getProperty(
                     "org.apache.jasper.compiler.Parser.STRICT_WHITESPACE",
-                    "true"));
+                    "true")).booleanValue();
     /**
      * The constructor
      */
     private Parser(ParserController pc, JspReader reader, boolean isTagFile,
-            boolean directivesOnly, JarResource jarResource) {
+            boolean directivesOnly, Jar jar) {
         this.parserController = pc;
         this.ctxt = pc.getJspCompilationContext();
         this.pageInfo = pc.getCompiler().getPageInfo();
@@ -95,13 +97,13 @@ class Parser implements TagConstants {
         this.scriptlessCount = 0;
         this.isTagFile = isTagFile;
         this.directivesOnly = directivesOnly;
-        this.jarResource = jarResource;
+        this.jar = jar;
         start = reader.mark();
     }
 
     /**
      * The main entry for Parser
-     * 
+     *
      * @param pc
      *            The ParseController, use for getting other objects in compiler
      *            and for parsing included pages
@@ -113,12 +115,11 @@ class Parser implements TagConstants {
      */
     public static Node.Nodes parse(ParserController pc, JspReader reader,
             Node parent, boolean isTagFile, boolean directivesOnly,
-            JarResource jarResource, String pageEnc, String jspConfigPageEnc,
+            Jar jar, String pageEnc, String jspConfigPageEnc,
             boolean isDefaultPageEncoding, boolean isBomPresent)
             throws JasperException {
 
-        Parser parser = new Parser(pc, reader, isTagFile, directivesOnly,
-                jarResource);
+        Parser parser = new Parser(pc, reader, isTagFile, directivesOnly, jar);
 
         Node.Root root = new Node.Root(reader.mark(), parent, false);
         root.setPageEncoding(pageEnc);
@@ -231,7 +232,7 @@ class Parser implements TagConstants {
         }
         watchString = watchString + quote;
 
-        String attrValue = parseAttributeValue(watchString, ignoreEL);
+        String attrValue = parseAttributeValue(qName, watchString, ignoreEL);
         attrs.addAttribute(uri, localName, qName, "CDATA", attrValue);
         return true;
     }
@@ -239,7 +240,7 @@ class Parser implements TagConstants {
     /**
      * Name ::= (Letter | '_' | ':') (Letter | Digit | '.' | '_' | '-' | ':')*
      */
-    private String parseName() throws JasperException {
+    private String parseName() {
         char ch = (char) reader.peekChar();
         if (Character.isLetter(ch) || ch == '_' || ch == ':') {
             StringBuilder buf = new StringBuilder();
@@ -262,29 +263,25 @@ class Parser implements TagConstants {
      * RTAttributeValueDouble ::= ((QuotedChar - '"')* - ((QuotedChar-'"')'%>"')
      * ('%>"' | TRANSLATION_ERROR)
      */
-    private String parseAttributeValue(String watch, boolean ignoreEL) throws JasperException {
-        boolean quoteAttributeEL = ctxt.getOptions().getQuoteAttributeEL();
+    private String parseAttributeValue(String qName, String watch, boolean ignoreEL) throws JasperException {
         Mark start = reader.mark();
-        // In terms of finding the end of the value, quoting EL is equivalent to
-        // ignoring it.
-        Mark stop = reader.skipUntilIgnoreEsc(watch, ignoreEL || quoteAttributeEL);
+        Mark stop = reader.skipUntilIgnoreEsc(watch, ignoreEL);
         if (stop == null) {
-            err.jspError(start, "jsp.error.attribute.unterminated", watch);
+            err.jspError(start, "jsp.error.attribute.unterminated", qName);
         }
 
         String ret = null;
         try {
             char quote = watch.charAt(watch.length() - 1);
-            
+
             // If watch is longer than 1 character this is a scripting
             // expression and EL is always ignored
             boolean isElIgnored =
                 pageInfo.isELIgnored() || watch.length() > 1;
-            
+
             ret = AttributeParser.getUnquoted(reader.getText(start, stop),
                     quote, isElIgnored,
-                    pageInfo.isDeferredSyntaxAllowedAsLiteral(),
-                    quoteAttributeEL);
+                    pageInfo.isDeferredSyntaxAllowedAsLiteral());
         } catch (IllegalArgumentException iae) {
             err.jspError(start, iae.getMessage());
         }
@@ -326,7 +323,7 @@ class Parser implements TagConstants {
         }
 
         try {
-            parserController.parse(file, parent, jarResource);
+            parserController.parse(file, parent, jar);
         } catch (FileNotFoundException ex) {
             err.jspError(start, "jsp.error.file.not.found", file);
         } catch (Exception ex) {
@@ -370,7 +367,7 @@ class Parser implements TagConstants {
      * Add a list of files. This is used for implementing include-prelude and
      * include-coda of jsp-config element in web.xml
      */
-    private void addInclude(Node parent, List<String> files) throws JasperException {
+    private void addInclude(Node parent, Collection<String> files) throws JasperException {
         if (files != null) {
             Iterator<String> iter = files.iterator();
             while (iter.hasNext()) {
@@ -415,20 +412,12 @@ class Parser implements TagConstants {
                                 .getCache().get(uri);
                     }
                     if (impl == null) {
-                        TldLocation location = ctxt.getTldLocation(uri);
+                        TldResourcePath tldResourcePath = ctxt.getTldResourcePath(uri);
                         impl = new TagLibraryInfoImpl(ctxt, parserController,
-                                pageInfo, prefix, uri, location, err,
-                                reader.mark());
+                                pageInfo, prefix, uri, tldResourcePath, err);
                         if (ctxt.getOptions().isCaching()) {
                             ctxt.getOptions().getCache().put(uri, impl);
                         }
-                    } else {
-                        // Current compilation context needs location of cached
-                        // tag files
-                        for (TagFileInfo info : impl.getTagFiles()) {
-                            ctxt.setTagFileJarResource(info.getPath(),
-                                    ctxt.getTagFileJarResource());
-                        }
                     }
                     pageInfo.addTaglib(uri, impl);
                 }
@@ -448,14 +437,15 @@ class Parser implements TagConstants {
             }
         }
 
-        new Node.TaglibDirective(attrs, start, parent);
+        @SuppressWarnings("unused")
+        Node unused = new Node.TaglibDirective(attrs, start, parent);
     }
 
     /*
      * Parses a directive with the following syntax: Directive ::= S? ( 'page'
      * PageDirective | 'include' IncludeDirective | 'taglib' TagLibDirective) S?
      * '%>'
-     * 
+     *
      * TagDirective ::= S? ('tag' PageDirective | 'include' IncludeDirective |
      * 'taglib' TagLibDirective) | 'attribute AttributeDirective | 'variable
      * VariableDirective S? '%>'
@@ -515,11 +505,11 @@ class Parser implements TagConstants {
 
     /*
      * Parses a directive with the following syntax:
-     * 
+     *
      * XMLJSPDirectiveBody ::= S? ( ( 'page' PageDirectiveAttrList S? ( '/>' | (
      * '>' S? ETag ) ) | ( 'include' IncludeDirectiveAttrList S? ( '/>' | ( '>'
      * S? ETag ) ) | <TRANSLATION_ERROR>
-     * 
+     *
      * XMLTagDefDirectiveBody ::= ( ( 'tag' TagDirectiveAttrList S? ( '/>' | (
      * '>' S? ETag ) ) | ( 'include' IncludeDirectiveAttrList S? ( '/>' | ( '>'
      * S? ETag ) ) | ( 'attribute' AttributeDirectiveAttrList S? ( '/>' | ( '>'
@@ -602,7 +592,8 @@ class Parser implements TagConstants {
      */
     private void parseAttributeDirective(Node parent) throws JasperException {
         Attributes attrs = parseAttributes();
-        new Node.AttributeDirective(attrs, start, parent);
+        @SuppressWarnings("unused")
+        Node unused = new Node.AttributeDirective(attrs, start, parent);
     }
 
     /*
@@ -611,7 +602,8 @@ class Parser implements TagConstants {
      */
     private void parseVariableDirective(Node parent) throws JasperException {
         Attributes attrs = parseAttributes();
-        new Node.VariableDirective(attrs, start, parent);
+        @SuppressWarnings("unused")
+        Node unused = new Node.VariableDirective(attrs, start, parent);
     }
 
     /*
@@ -624,7 +616,9 @@ class Parser implements TagConstants {
             err.jspError(start, "jsp.error.unterminated", "&lt;%--");
         }
 
-        new Node.Comment(reader.getText(start, stop), start, parent);
+        @SuppressWarnings("unused")
+        Node unused =
+                new Node.Comment(reader.getText(start, stop), start, parent);
     }
 
     /*
@@ -637,8 +631,9 @@ class Parser implements TagConstants {
             err.jspError(start, "jsp.error.unterminated", "&lt;%!");
         }
 
-        new Node.Declaration(parseScriptText(reader.getText(start, stop)),
-                start, parent);
+        @SuppressWarnings("unused")
+        Node unused = new Node.Declaration(
+                parseScriptText(reader.getText(start, stop)), start, parent);
     }
 
     /*
@@ -664,7 +659,8 @@ class Parser implements TagConstants {
                             "&lt;jsp:declaration&gt;");
                 }
                 text = parseScriptText(reader.getText(start, stop));
-                new Node.Declaration(text, start, parent);
+                @SuppressWarnings("unused")
+                Node unused = new Node.Declaration(text, start, parent);
                 if (reader.matches("![CDATA[")) {
                     start = reader.mark();
                     stop = reader.skipUntil("]]>");
@@ -672,7 +668,8 @@ class Parser implements TagConstants {
                         err.jspError(start, "jsp.error.unterminated", "CDATA");
                     }
                     text = parseScriptText(reader.getText(start, stop));
-                    new Node.Declaration(text, start, parent);
+                    @SuppressWarnings("unused")
+                    Node unused2 = new Node.Declaration(text, start, parent);
                 } else {
                     break;
                 }
@@ -695,8 +692,9 @@ class Parser implements TagConstants {
             err.jspError(start, "jsp.error.unterminated", "&lt;%=");
         }
 
-        new Node.Expression(parseScriptText(reader.getText(start, stop)),
-                start, parent);
+        @SuppressWarnings("unused")
+        Node unused = new Node.Expression(
+                parseScriptText(reader.getText(start, stop)), start, parent);
     }
 
     /*
@@ -720,7 +718,8 @@ class Parser implements TagConstants {
                             "&lt;jsp:expression&gt;");
                 }
                 text = parseScriptText(reader.getText(start, stop));
-                new Node.Expression(text, start, parent);
+                @SuppressWarnings("unused")
+                Node unused = new Node.Expression(text, start, parent);
                 if (reader.matches("![CDATA[")) {
                     start = reader.mark();
                     stop = reader.skipUntil("]]>");
@@ -728,7 +727,8 @@ class Parser implements TagConstants {
                         err.jspError(start, "jsp.error.unterminated", "CDATA");
                     }
                     text = parseScriptText(reader.getText(start, stop));
-                    new Node.Expression(text, start, parent);
+                    @SuppressWarnings("unused")
+                    Node unused2 = new Node.Expression(text, start, parent);
                 } else {
                     break;
                 }
@@ -741,7 +741,8 @@ class Parser implements TagConstants {
     }
 
     /*
-     * ELExpressionBody (following "${" to first unquoted "}")
+     * ELExpressionBody. Starts with "#{" or "${".  Ends with "}".
+     * See JspReader.skipELExpression().
      */
     private void parseELExpression(Node parent, char type)
             throws JasperException {
@@ -751,7 +752,9 @@ class Parser implements TagConstants {
             err.jspError(start, "jsp.error.unterminated", type + "{");
         }
 
-        new Node.ELExpression(type, reader.getText(start, last), start, parent);
+        @SuppressWarnings("unused")
+        Node unused = new Node.ELExpression(type, reader.getText(start, last),
+                start, parent);
     }
 
     /*
@@ -764,8 +767,9 @@ class Parser implements TagConstants {
             err.jspError(start, "jsp.error.unterminated", "&lt;%");
         }
 
-        new Node.Scriptlet(parseScriptText(reader.getText(start, stop)), start,
-                parent);
+        @SuppressWarnings("unused")
+        Node unused = new Node.Scriptlet(
+                parseScriptText(reader.getText(start, stop)), start, parent);
     }
 
     /*
@@ -789,7 +793,8 @@ class Parser implements TagConstants {
                             "&lt;jsp:scriptlet&gt;");
                 }
                 text = parseScriptText(reader.getText(start, stop));
-                new Node.Scriptlet(text, start, parent);
+                @SuppressWarnings("unused")
+                Node unused = new Node.Scriptlet(text, start, parent);
                 if (reader.matches("![CDATA[")) {
                     start = reader.mark();
                     stop = reader.skipUntil("]]>");
@@ -797,7 +802,8 @@ class Parser implements TagConstants {
                         err.jspError(start, "jsp.error.unterminated", "CDATA");
                     }
                     text = parseScriptText(reader.getText(start, stop));
-                    new Node.Scriptlet(text, start, parent);
+                    @SuppressWarnings("unused")
+                    Node unused2 = new Node.Scriptlet(text, start, parent);
                 } else {
                     break;
                 }
@@ -829,13 +835,13 @@ class Parser implements TagConstants {
 
     /*
      * For Include: StdActionContent ::= Attributes ParamBody
-     * 
+     *
      * ParamBody ::= EmptyBody | ( '>' S? ( '<jsp:attribute' NamedAttributes )? '<jsp:body'
      * (JspBodyParam | <TRANSLATION_ERROR> ) S? ETag ) | ( '>' S? Param* ETag )
-     * 
+     *
      * EmptyBody ::= '/>' | ( '>' ETag ) | ( '>' S? '<jsp:attribute'
      * NamedAttributes ETag )
-     * 
+     *
      * JspBodyParam ::= S? '>' Param* '</jsp:body>'
      */
     private void parseInclude(Node parent) throws JasperException {
@@ -956,23 +962,23 @@ class Parser implements TagConstants {
      * since the syntax is identical (the only thing that differs substantially
      * is how to process the body, and thus we accept the body type as a
      * parameter).
-     * 
+     *
      * OptionalBody ::= EmptyBody | ActionBody
-     * 
+     *
      * ScriptlessOptionalBody ::= EmptyBody | ScriptlessActionBody
-     * 
+     *
      * TagDependentOptionalBody ::= EmptyBody | TagDependentActionBody
-     * 
+     *
      * EmptyBody ::= '/>' | ( '>' ETag ) | ( '>' S? '<jsp:attribute'
      * NamedAttributes ETag )
-     * 
+     *
      * ActionBody ::= JspAttributeAndBody | ( '>' Body ETag )
-     * 
+     *
      * ScriptlessActionBody ::= JspAttributeAndBody | ( '>' ScriptlessBody ETag )
-     * 
+     *
      * TagDependentActionBody ::= JspAttributeAndBody | ( '>' TagDependentBody
      * ETag )
-     * 
+     *
      */
     private void parseOptionalBody(Node parent, String tag, String bodyType)
             throws JasperException {
@@ -999,7 +1005,7 @@ class Parser implements TagConstants {
     /**
      * Attempts to parse 'JspAttributeAndBody' production. Returns true if it
      * matched, or false if not. Assumes EmptyBody is okay as well.
-     * 
+     *
      * JspAttributeAndBody ::= ( '>' # S? ( '<jsp:attribute' NamedAttributes )? '<jsp:body' (
      * JspBodyBody | <TRANSLATION_ERROR> ) S? ETag )
      */
@@ -1059,14 +1065,14 @@ class Parser implements TagConstants {
 
     /*
      * For Plugin: StdActionContent ::= Attributes PluginBody
-     * 
+     *
      * PluginBody ::= EmptyBody | ( '>' S? ( '<jsp:attribute' NamedAttributes )? '<jsp:body' (
      * JspBodyPluginTags | <TRANSLATION_ERROR> ) S? ETag ) | ( '>' S? PluginTags
      * ETag )
-     * 
+     *
      * EmptyBody ::= '/>' | ( '>' ETag ) | ( '>' S? '<jsp:attribute'
      * NamedAttributes ETag )
-     * 
+     *
      */
     private void parsePlugin(Node parent) throws JasperException {
         Attributes attrs = parseAttributes();
@@ -1150,26 +1156,27 @@ class Parser implements TagConstants {
 
     /*
      * # '<' CustomAction CustomActionBody
-     * 
+     *
      * CustomAction ::= TagPrefix ':' CustomActionName
-     * 
+     *
      * TagPrefix ::= Name
-     * 
+     *
      * CustomActionName ::= Name
-     * 
+     *
      * CustomActionBody ::= ( Attributes CustomActionEnd ) | <TRANSLATION_ERROR>
-     * 
+     *
      * Attributes ::= ( S Attribute )* S?
-     * 
+     *
      * CustomActionEnd ::= CustomActionTagDependent | CustomActionJSPContent |
      * CustomActionScriptlessContent
-     * 
+     *
      * CustomActionTagDependent ::= TagDependentOptionalBody
-     * 
+     *
      * CustomActionJSPContent ::= OptionalBody
-     * 
+     *
      * CustomActionScriptlessContent ::= ScriptlessOptionalBody
      */
+    @SuppressWarnings("null") // tagFileInfo can't be null after initial test
     private boolean parseCustomTag(Node parent) throws JasperException {
 
         if (reader.peekChar() != '<') {
@@ -1232,11 +1239,13 @@ class Parser implements TagConstants {
         // Parse 'CustomActionEnd' production:
         if (reader.matches("/>")) {
             if (tagInfo != null) {
-                new Node.CustomTag(tagName, prefix, shortTagName, uri, attrs,
-                        start, parent, tagInfo, tagHandlerClass);
+                @SuppressWarnings("unused")
+                Node unused = new Node.CustomTag(tagName, prefix, shortTagName,
+                        uri, attrs, start, parent, tagInfo, tagHandlerClass);
             } else {
-                new Node.CustomTag(tagName, prefix, shortTagName, uri, attrs,
-                        start, parent, tagFileInfo);
+                @SuppressWarnings("unused")
+                Node unused = new Node.CustomTag(tagName, prefix, shortTagName,
+                        uri, attrs, start, parent, tagFileInfo);
             }
             return true;
         }
@@ -1277,7 +1286,7 @@ class Parser implements TagConstants {
      *       '\${' for '${' and '\#{' for '#{'. We are processing JSP template
      *       test here so the JSP escapes apply.
      */
-    private void parseTemplateText(Node parent) throws JasperException {
+    private void parseTemplateText(Node parent) {
 
         if (!reader.hasMoreInput())
             return;
@@ -1354,25 +1363,25 @@ class Parser implements TagConstants {
                     String text = reader.getText(start, stop);
                     ttext.write(text, 0, text.length());
                 } else if (ch == '\\') {
-                    int next = reader.peekChar();
+                    int next = reader.peekChar(0);
                     if (next == '$' || next =='#') {
                         ttext.write(reader.nextChar());
                     } else {
                         ttext.write('\\');
                     }
                 } else if (ch == '$' || ch == '#') {
-                    if (reader.peekChar() == '{') {
+                    if (reader.peekChar(0) == '{') {
                         // Swallow the '{'
                         reader.nextChar();
-                        
+
                         // Create a template text node
                         @SuppressWarnings("unused")
                         Node unused = new Node.TemplateText(
                                 ttext.toString(), start, parent);
-    
+
                         // Mark and parse the EL expression and create its node:
                         parseELExpression(parent, (char) ch);
-    
+
                         start = reader.mark();
                         ttext.reset();
                     } else {
@@ -1592,8 +1601,9 @@ class Parser implements TagConstants {
         if (bodyEnd == null) {
             err.jspError(start, "jsp.error.unterminated", "&lt;" + tag);
         }
-        new Node.TemplateText(reader.getText(bodyStart, bodyEnd), bodyStart,
-                parent);
+        @SuppressWarnings("unused")
+        Node unused = new Node.TemplateText(reader.getText(bodyStart, bodyEnd),
+                bodyStart, parent);
     }
 
     /*
@@ -1762,7 +1772,6 @@ class Parser implements TagConstants {
     }
 
     private void parseFileDirectives(Node parent) throws JasperException {
-        reader.setSingleFile(true);
         reader.skipUntil("<");
         while (reader.hasMoreInput()) {
             start = reader.mark();
