@@ -23,9 +23,7 @@ import java.nio.ByteBuffer;
 /** Socket
  *
  * @author Mladen Turk
- *
  */
-
 public class Socket {
 
     /* Standard socket defines */
@@ -119,7 +117,7 @@ public class Socket {
 
     /**
      * Shutdown either reading, writing, or both sides of a socket.
-     * <br />
+     * <br>
      * This does not actually close the socket descriptor, it just
      *      controls which calls are still valid on the socket.
      * @param thesocket The socket to close
@@ -251,12 +249,43 @@ public class Socket {
      */
     public static native int sendb(long sock, ByteBuffer buf,
                                    int offset, int len);
+
+    /**
+     * Send data over a network without retry
+     * <PRE>
+     * This functions acts like a blocking write by default.  To change
+     * this behavior, use apr_socket_timeout_set() or the APR_SO_NONBLOCK
+     * socket option.
+     *
+     * It is possible for both bytes to be sent and an error to be returned.
+     *
+     * </PRE>
+     * @param sock The socket to send the data over.
+     * @param buf The Byte buffer which contains the data to be sent.
+     * @param offset The offset within the buffer array of the first buffer from
+     *               which bytes are to be retrieved; must be non-negative
+     *               and no larger than buf.length
+     * @param len The maximum number of buffers to be accessed; must be non-negative
+     *            and no larger than buf.length - offset
+     * @return The number of bytes send.
+     *
+     */
+    public static native int sendib(long sock, ByteBuffer buf,
+                                    int offset, int len);
+
     /**
      * Send data over a network using internally set ByteBuffer
      */
     public static native int sendbb(long sock,
                                    int offset, int len);
 
+    /**
+     * Send data over a network using internally set ByteBuffer
+     * without internal retry.
+     */
+    public static native int sendibb(long sock,
+                                     int offset, int len);
+
     /**
      * Send multiple packets of data over a network.
      * <PRE>
@@ -350,12 +379,19 @@ public class Socket {
      * @param buf The buffer to store the data in.
      * @param offset Offset in the byte buffer.
      * @param nbytes The number of bytes to read (-1) for full array.
-     * @return the number of bytes received.
+     * @return If >= 0, the return value is the number of bytes read. Note a
+     *         non-blocking read with no data current available will return
+     *         {@link Status#EAGAIN} and EOF will return {@link Status#APR_EOF}.
      */
     public static native int recvb(long sock, ByteBuffer buf,
                                    int offset, int nbytes);
     /**
-     * Read data from a network using internally set ByteBuffer
+     * Read data from a network using internally set ByteBuffer.
+     *
+     * @return If > 0, the return value is the number of bytes read. If == 0,
+     *         the return value indicates EOF and if < 0 the return value is the
+     *         error code. Note a non-blocking read with no data current
+     *         available will return {@link Status#EAGAIN} not zero.
      */
     public static native int recvbb(long sock,
                                     int offset, int nbytes);
@@ -451,10 +487,10 @@ public class Socket {
      * @param sock The socket to set up.
      * @param t Value for the timeout in microseconds.
      * <PRE>
-     * t > 0  -- read and write calls return APR_TIMEUP if specified time
-     *           elapsess with no data read or written
+     * t &gt; 0  -- read and write calls return APR_TIMEUP if specified time
+     *           elapses with no data read or written
      * t == 0 -- read and write calls never block
-     * t < 0  -- read and write calls block
+     * t &lt; 0  -- read and write calls block
      * </PRE>
      */
     public static native int timeoutSet(long sock, long t);
@@ -470,7 +506,7 @@ public class Socket {
     /**
      * Send a file from an open file descriptor to a socket, along with
      * optional headers and trailers.
-     * <br />
+     * <br>
      * This functions acts like a blocking write by default.  To change
      *         this behavior, use apr_socket_timeout_set() or the
      *         APR_SO_NONBLOCK socket option.
@@ -506,8 +542,8 @@ public class Socket {
         throws Exception;
 
     /**
-     * Private method for geting the socket struct members
-     * @param socket The soocket to use
+     * Private method for getting the socket struct members
+     * @param socket The socket to use
      * @param what Struct member to obtain
      * <PRE>
      * SOCKET_GET_POOL  - The socket pool
@@ -515,7 +551,7 @@ public class Socket {
      * SOCKET_GET_APRS  - APR socket
      * SOCKET_GET_TYPE  - Socket type
      * </PRE>
-     * @return The stucture member address
+     * @return The structure member address
      */
     private static native long get(long socket, int what);
 
@@ -536,4 +572,20 @@ public class Socket {
      * @param buf The ByteBuffer
      */
     public static native void setrbb(long sock, ByteBuffer buf);
+
+    /**
+     * Set the data associated with the current socket.
+     * @param sock The currently open socket.
+     * @param data The user data to associate with the socket.
+     * @param key The key to associate with the data.
+     */
+      public static native int dataSet(long sock, String key, Object data);
+
+    /**
+     * Return the data associated with the current socket
+     * @param key The key to associate with the user data.
+     * @param sock The currently open socket.
+     * @return Data or null in case of error.
+     */
+     public static native Object dataGet(long sock, String key);
 }
