@@ -366,16 +366,16 @@ public class SecureNioChannel extends NioChannel  {
     }
 
     /**
-     * Sends a SSL close message, will not physically close the connection here.<br>
-     * To close the connection, you could do something like
+     * Sends a SSL close message, will not physically close the connection here.
+     * <br>To close the connection, you could do something like
      * <pre><code>
      *   close();
      *   while (isOpen() &amp;&amp; !myTimeoutFunction()) Thread.sleep(25);
      *   if ( isOpen() ) close(true); //forces a close if you timed out
      * </code></pre>
      * @throws IOException if an I/O error occurs
-     * @throws IOException if there is data on the outgoing network buffer and we are unable to flush it
-     * TODO Implement this java.io.Closeable method
+     * @throws IOException if there is data on the outgoing network buffer and
+     *                     we are unable to flush it
      */
     @Override
     public void close() throws IOException {
@@ -425,10 +425,11 @@ public class SecureNioChannel extends NioChannel  {
      * Reads a sequence of bytes from this channel into the given buffer.
      *
      * @param dst The buffer into which bytes are to be transferred
-     * @return The number of bytes read, possibly zero, or <tt>-1</tt> if the channel has reached end-of-stream
+     * @return The number of bytes read, possibly zero, or <tt>-1</tt> if the
+     *         channel has reached end-of-stream
      * @throws IOException If some other I/O error occurs
-     * @throws IllegalArgumentException if the destination buffer is different than bufHandler.getReadBuffer()
-     * TODO Implement this java.nio.channels.ReadableByteChannel method
+     * @throws IllegalArgumentException if the destination buffer is different
+     *                                  than bufHandler.getReadBuffer()
      */
     @Override
     public int read(ByteBuffer dst) throws IOException {
@@ -483,37 +484,32 @@ public class SecureNioChannel extends NioChannel  {
      * @param src The buffer from which bytes are to be retrieved
      * @return The number of bytes written, possibly zero
      * @throws IOException If some other I/O error occurs
-     * TODO Implement this java.nio.channels.WritableByteChannel method
      */
     @Override
     public int write(ByteBuffer src) throws IOException {
         checkInterruptStatus();
-        if ( src == this.netOutBuffer ) {
+        if (src == this.netOutBuffer) {
             //we can get here through a recursive call
             //by using the NioBlockingSelector
             int written = sc.write(src);
             return written;
         } else {
-            //make sure we can handle expand, and that we only use one buffer
-            if ( (!this.isSendFile()) && (src != bufHandler.getWriteBuffer()) ) throw new IllegalArgumentException(sm.getString("channel.nio.ssl.invalidBuffer"));
-            //are we closing or closed?
-            if ( closing || closed) throw new IOException(sm.getString("channel.nio.ssl.closing"));
-
-            //the number of bytes written
-            int written = 0;
-
-            if (!flush(netOutBuffer)) {
-                //we haven't emptied out the buffer yet
-                return written;
+            // Are we closing or closed?
+            if (closing || closed) {
+                throw new IOException(sm.getString("channel.nio.ssl.closing"));
             }
 
-            /*
-             * The data buffer is empty, we can reuse the entire buffer.
-             */
+            if (!flush(netOutBuffer)) {
+                // We haven't emptied out the buffer yet
+                return 0;
+            }
+
+            // The data buffer is empty, we can reuse the entire buffer.
             netOutBuffer.clear();
 
             SSLEngineResult result = sslEngine.wrap(src, netOutBuffer);
-            written = result.bytesConsumed();
+            // The number of bytes written
+            int written = result.bytesConsumed();
             netOutBuffer.flip();
 
             if (result.getStatus() == Status.OK) {
@@ -522,7 +518,7 @@ public class SecureNioChannel extends NioChannel  {
                 throw new IOException(sm.getString("channel.nio.ssl.wrapFail", result.getStatus()));
             }
 
-            //force a flush
+            // Force a flush
             flush(netOutBuffer);
 
             return written;
