@@ -358,14 +358,30 @@ public class TestCipher {
      */
     private static Set<String> OPENSSL_UNMAPPED_ORACLE =
             Collections.unmodifiableSet(new HashSet<>(Arrays.asList(
+                    "AES128-CCM+TLSv1.2",
+                    "AES128-CCM8+TLSv1.2",
+                    "AES256-CCM+TLSv1.2",
+                    "AES256-CCM8+TLSv1.2",
                     "DES-CBC-MD5+SSLv2",
                     "DES-CBC3-MD5+SSLv2",
+                    "DHE-PSK-AES128-CCM+TLSv1.2",
+                    "DHE-PSK-AES128-CCM8+TLSv1.2",
+                    "DHE-PSK-AES256-CCM+TLSv1.2",
+                    "DHE-PSK-AES256-CCM8+TLSv1.2",
                     "DHE-PSK-CAMELLIA128-SHA256+SSLv3",
                     "DHE-PSK-CAMELLIA256-SHA384+SSLv3",
+                    "DHE-RSA-AES128-CCM+TLSv1.2",
+                    "DHE-RSA-AES128-CCM8+TLSv1.2",
+                    "DHE-RSA-AES256-CCM+TLSv1.2",
+                    "DHE-RSA-AES256-CCM8+TLSv1.2",
                     "ECDH-ECDSA-CAMELLIA128-SHA256+TLSv1.2",
                     "ECDH-ECDSA-CAMELLIA256-SHA384+TLSv1.2",
                     "ECDH-RSA-CAMELLIA128-SHA256+TLSv1.2",
                     "ECDH-RSA-CAMELLIA256-SHA384+TLSv1.2",
+                    "ECDHE-ECDSA-AES128-CCM+TLSv1.2",
+                    "ECDHE-ECDSA-AES128-CCM8+TLSv1.2",
+                    "ECDHE-ECDSA-AES256-CCM+TLSv1.2",
+                    "ECDHE-ECDSA-AES256-CCM8+TLSv1.2",
                     "ECDHE-ECDSA-CAMELLIA128-SHA256+TLSv1.2",
                     "ECDHE-ECDSA-CAMELLIA256-SHA384+TLSv1.2",
                     "ECDHE-PSK-CAMELLIA128-SHA256+SSLv3",
@@ -375,6 +391,10 @@ public class TestCipher {
                     "EXP-RC2-CBC-MD5+SSLv2",
                     "EXP-RC4-MD5+SSLv2",
                     "IDEA-CBC-MD5+SSLv2",
+                    "PSK-AES128-CCM+TLSv1.2",
+                    "PSK-AES128-CCM8+TLSv1.2",
+                    "PSK-AES256-CCM+TLSv1.2",
+                    "PSK-AES256-CCM8+TLSv1.2",
                     "PSK-CAMELLIA128-SHA256+SSLv3",
                     "PSK-CAMELLIA256-SHA384+SSLv3",
                     "RC2-CBC-MD5+SSLv2",
@@ -522,6 +542,10 @@ public class TestCipher {
                     "ADH-CAMELLIA128-SHA256+TLSv1.2",
                     "ADH-CAMELLIA256-SHA256+TLSv1.2",
                     "ADH-SEED-SHA+SSLv3",
+                    "AES128-CCM+TLSv1.2",
+                    "AES128-CCM8+TLSv1.2",
+                    "AES256-CCM+TLSv1.2",
+                    "AES256-CCM8+TLSv1.2",
                     "CAMELLIA128-SHA+SSLv3",
                     "CAMELLIA256-SHA+SSLv3",
                     "CAMELLIA128-SHA256+TLSv1.2",
@@ -562,9 +586,13 @@ public class TestCipher {
                     "DHE-PSK-3DES-EDE-CBC-SHA+SSLv3",
                     "DHE-PSK-AES128-CBC-SHA+SSLv3",
                     "DHE-PSK-AES128-CBC-SHA256+SSLv3",
+                    "DHE-PSK-AES128-CCM+TLSv1.2",
+                    "DHE-PSK-AES128-CCM8+TLSv1.2",
                     "DHE-PSK-AES128-GCM-SHA256+TLSv1.2",
                     "DHE-PSK-AES256-CBC-SHA+SSLv3",
                     "DHE-PSK-AES256-CBC-SHA384+SSLv3",
+                    "DHE-PSK-AES256-CCM+TLSv1.2",
+                    "DHE-PSK-AES256-CCM8+TLSv1.2",
                     "DHE-PSK-AES256-GCM-SHA384+TLSv1.2",
                     "DHE-PSK-CAMELLIA128-SHA256+SSLv3",
                     "DHE-PSK-CAMELLIA256-SHA384+SSLv3",
@@ -572,6 +600,10 @@ public class TestCipher {
                     "DHE-PSK-NULL-SHA256+SSLv3",
                     "DHE-PSK-NULL-SHA384+SSLv3",
                     "DHE-PSK-RC4-SHA+SSLv3",
+                    "DHE-RSA-AES128-CCM+TLSv1.2",
+                    "DHE-RSA-AES128-CCM8+TLSv1.2",
+                    "DHE-RSA-AES256-CCM+TLSv1.2",
+                    "DHE-RSA-AES256-CCM8+TLSv1.2",
                     "DHE-RSA-CAMELLIA128-SHA+SSLv3",
                     "DHE-RSA-CAMELLIA128-SHA256+TLSv1.2",
                     "DHE-RSA-CAMELLIA256-SHA+SSLv3",
@@ -581,6 +613,10 @@ public class TestCipher {
                     "ECDH-ECDSA-CAMELLIA256-SHA384+TLSv1.2",
                     "ECDH-RSA-CAMELLIA128-SHA256+TLSv1.2",
                     "ECDH-RSA-CAMELLIA256-SHA384+TLSv1.2",
+                    "ECDHE-ECDSA-AES128-CCM+TLSv1.2",
+                    "ECDHE-ECDSA-AES128-CCM8+TLSv1.2",
+                    "ECDHE-ECDSA-AES256-CCM+TLSv1.2",
+                    "ECDHE-ECDSA-AES256-CCM8+TLSv1.2",
                     "ECDHE-ECDSA-CAMELLIA128-SHA256+TLSv1.2",
                     "ECDHE-ECDSA-CAMELLIA256-SHA384+TLSv1.2",
                     "ECDHE-PSK-3DES-EDE-CBC-SHA+SSLv3",
@@ -605,9 +641,13 @@ public class TestCipher {
                     "PSK-3DES-EDE-CBC-SHA+SSLv3",
                     "PSK-AES128-CBC-SHA+SSLv3",
                     "PSK-AES128-CBC-SHA256+SSLv3",
+                    "PSK-AES128-CCM+TLSv1.2",
+                    "PSK-AES128-CCM8+TLSv1.2",
                     "PSK-AES128-GCM-SHA256+TLSv1.2",
                     "PSK-AES256-CBC-SHA+SSLv3",
                     "PSK-AES256-CBC-SHA384+SSLv3",
+                    "PSK-AES256-CCM+TLSv1.2",
+                    "PSK-AES256-CCM8+TLSv1.2",
                     "PSK-AES256-GCM-SHA384+TLSv1.2",
                     "PSK-CAMELLIA128-SHA256+SSLv3",
                     "PSK-CAMELLIA256-SHA384+SSLv3",
