@@ -64,6 +64,7 @@ import org.apache.tomcat.websocket.server.WsContextListener;
 
 public class TestWsWebSocketContainer extends TomcatBaseTest {
 
+    private static final String MESSAGE_EMPTY = "";
     private static final String MESSAGE_STRING_1 = "qwerty";
     private static final String MESSAGE_TEXT_4K;
     private static final byte[] MESSAGE_BINARY_4K = new byte[4096];
@@ -926,7 +927,30 @@ public class TestWsWebSocketContainer extends TomcatBaseTest {
 
 
     @Test
-    public void testPerMessageDefalteClient() throws Exception {
+    public void testPerMessageDefalteClient01() throws Exception {
+        doTestPerMessageDefalteClient(MESSAGE_STRING_1, 1);
+    }
+
+
+    @Test
+    public void testPerMessageDefalteClient02() throws Exception {
+        doTestPerMessageDefalteClient(MESSAGE_EMPTY, 1);
+    }
+
+
+    @Test
+    public void testPerMessageDefalteClient03() throws Exception {
+        doTestPerMessageDefalteClient(MESSAGE_STRING_1, 2);
+    }
+
+
+    @Test
+    public void testPerMessageDefalteClient04() throws Exception {
+        doTestPerMessageDefalteClient(MESSAGE_EMPTY, 2);
+    }
+
+
+    private void doTestPerMessageDefalteClient(String msg, int count) throws Exception {
         Tomcat tomcat = getTomcatInstance();
         // No file system docBase required
         Context ctx = tomcat.addContext("", null);
@@ -950,19 +974,17 @@ public class TestWsWebSocketContainer extends TomcatBaseTest {
                 clientConfig,
                 new URI("ws://localhost:" + getPort() +
                         TesterEchoServer.Config.PATH_ASYNC));
-        CountDownLatch latch = new CountDownLatch(1);
-        BasicText handler = new BasicText(latch);
+        CountDownLatch latch = new CountDownLatch(count);
+        BasicText handler = new BasicText(latch, msg);
         wsSession.addMessageHandler(handler);
-        wsSession.getBasicRemote().sendText(MESSAGE_STRING_1);
+        for (int i = 0; i < count; i++) {
+            wsSession.getBasicRemote().sendText(msg);
+        }
 
         boolean latchResult = handler.getLatch().await(10, TimeUnit.SECONDS);
 
         Assert.assertTrue(latchResult);
 
-        Queue<String> messages = handler.getMessages();
-        Assert.assertEquals(1, messages.size());
-        Assert.assertEquals(MESSAGE_STRING_1, messages.peek());
-
         ((WsWebSocketContainer) wsContainer).destroy();
     }
 }
